﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;

namespace ProcessRubyScript
{
    class RubyExpressions
    {
        public static List<string> GetScriptExpressions(string ruby_file)
        {
            /*
             * This is currently set to retrieve Ruby method names and the starting and ending
             * line numbers followed by separating characters '::' and then the method code
             * with all whitespace and comment lines removed. We are not looking for embedded comments
             * inside a line of code due to complexity, for example "#{tmp}" would not be a 
             * comment but the regex would be quite complicated plus I am not aware of any DCS
             * ever adding an embedded comment.
             * 
             * Since we are not including comment lines, the starting line number is where we
             * detect the 'def'. However, the ending line number does reflect ALL lines in the
             * Ruby script, even the empty lines.
            */
            List<string> ExpressionList = new List<string>();
            
            string script_ruby_file = ruby_file.Replace('\\', '/');

            string Script = "def CompleteExpression?(str, b)\nold_verbose = $VERBOSE\n$VERBOSE = nil\ncatch(:valid) do\neval(\"BEGIN{throw :valid}\\n#{str}\", b)\nend\nstr !~ /[,\\\\]\\s*\\z/\nrescue SyntaxError => error\nfalse\nensure\n$VERBOSE = old_verbose\nend\n";

            Script += "def get_binding(param)\nreturn binding\nend\n";
            Script += "file = \"" + script_ruby_file + "\"\n";
            Script += "fileStr = IO.read(file)\n";
            Script += "fileStrBinding = get_binding(fileStr)\n";
            Script += "str = \"\"\n";
            Script += "lnStart = 0\n";
            Script += "lnCnt = 1\n";
            Script += "start = false\n";
            Script += "fileStr.each do |line|\n";
            Script += "start = true if (line =~ /\\A\\$\\\\=\"\\\\x03\\\\n\"\\Z/)\n";
            Script += "if (start)\nif ( (line =~ /\\A[ \\t]*#/) == nil )\n";
            Script += "lnStart = lnCnt if (lnStart == 0)\nstr += line\n";
            Script += "if (CompleteExpression?(str, fileStrBinding))\n";
            Script += "if (str =~ /\\A[ \\t]*def[ \\t]+(\\w+)/)\n";
            // Using 254.chr to separate the method body from the other parts.
            Script += "print \"#{$1}:#{lnStart} #{lnCnt}#{254.chr}#{str.gsub(/\\s+/,'')}\\n\"\nprint \"\\x03\\n\"\nend\n";
            Script += "lnStart = 0\nstr = \"\"\nend\nend\nend\nlnCnt += 1\nend\n";

            Process p = new Process();
            p.StartInfo.UseShellExecute = false;
            p.StartInfo.RedirectStandardOutput = true;
            p.StartInfo.RedirectStandardInput = true;
            p.StartInfo.CreateNoWindow = true;

            // This is what Joe A. was using when running for a different project on his computer.
            //p.StartInfo.FileName = @"..\Ruby187\ruby.exe";

            //Below works at work on MY computer only.
            //p.StartInfo.FileName = @"c:\Ruby186\ruby\bin\rubyw.exe";

            /*
             * Below did not work at work on MY computer:
             * c:/ruby/lib/ruby/1.8/i386-mingw32/rbconfig.rb is at 1.8.6 but
             * c:\ruby\bin\rubyw.exe is ver. 1.8.7. [This line does work at home however, 
             * both are 1.8.6.] USE THIS LINE FOR DCS TESTING AT WORK AND FOR RELEASING TO PRODUCTION,
             * those computers are at 1.8.6 for both c:/ruby/lib... and c:\ruby\bin\rubyw.exe
            */
            p.StartInfo.FileName = @"c:\ruby\bin\rubyw.exe";

            p.Start();

            p.StandardInput.Write(Script);
            p.StandardInput.Flush();
            p.StandardInput.Close();

            // To avoid deadlocks, always read the output stream first and then wait.
            string output = p.StandardOutput.ReadToEnd();
            char etx = (char)3;
            string[] expressions = output.Split(new char[] { etx });

            for (int i = 0; i < expressions.Length; ++i)
            {
                string exp = expressions[i].Trim();

                if (exp.Length > 0)
                    ExpressionList.Add(exp);
            }

            p.WaitForExit();
            
            return ExpressionList;
        }
    }
}